// A brute-force strategy: Comb the whole board, row by row and column by
// column, trying all possible combinations of values.  For each possible
// value, set it and try to solve the rest of the board.  Recursion is used
// to prune paths that lead to invalid board states.

// This method is guaranteed to work, but can take quite a while
// to produce a solution.

#include "strategy3.h"

#include <iostream>

bool SudokuStrategy3::operator()(SudokuBoard& board)
{
	try
	{
		int test_count = 0;
		SudokuBoard* ret_board;
		ret_board = solve(board, test_count);

		if(ret_board != NULL)
		{
			std::cout << "Brute force successful after "
				  << test_count << " value tests." << std::endl;
			board = *ret_board;
			delete ret_board;
		}
		else
			return false;
	}
	catch(...)
	{
		return false;
	}

	return true;
}

// The basic strategy is to find an empty cell, fill it with a
// potential value, and then solve the rest of the board recursively.
// Returns a valid solution if one is found, or throws an
// exception if a dead end (no more valid values can be placed)
// or invalid state is reached.

SudokuBoard* SudokuStrategy3::solve(SudokuBoard& board, int& test_count)
{
	// Get the next empty cell
	int col, row;
	try
	{
		board.next_empty(col, row);
	}
	catch(int)
	{
		std::cout << "Board is full without solution" << std::endl;
		return NULL;
	}

	// Try all possible values in that cell
	for(int i = 1; i <= SQUARE_VALUE_MAX; i++)
	{
		SudokuBoard board_copy = board;
		test_count++;
		if(board_copy.is_valid(col, row, i))
		{
			board_copy.set_value(col, row, i);
			if(board_copy.is_solved())
			{
				// Yay, a solution!
				return new SudokuBoard(board_copy);
			}

			// Now try solving the rest of the board
			SudokuBoard *ret_board;
			ret_board = solve(board_copy, test_count);

			// If we got a board back, it's a solution, otherwise
			// continue trying possible values
			if(ret_board)
				return ret_board;
		}
	}

	// Exhausted all possible values for that cell without a solution,
	// so we've hit a dead end.
	return NULL;
}
