#include "sudoku.h"

#include <fstream>
#include <iostream>

SudokuSquare::SudokuSquare()
{
	for(int i = 0; i < SQUARE_SIZE; i++)
	{
		for(int j = 0; j < SQUARE_SIZE; j++)
		{
			square_data[i][j] = -1;
		}
	}
}

bool SudokuSquare::is_present(int value) const
{
	for(int i = 0; i < SQUARE_SIZE; i++)
	{
		for(int j = 0; j < SQUARE_SIZE; j++)
		{
			if(square_data[i][j] == value)
				return true;
		}
	}
	return false;
}

void SudokuBoard::set_value(int col, int row, int value)
{
	int sqrow, sqcol, sqirow, sqicol;

	remap_coords(col, row, sqcol, sqrow, sqicol, sqirow);
	squares[sqcol][sqrow].square_data[sqicol][sqirow] = value;
}

int SudokuBoard::get_value(int col, int row) const
{
	int sqrow, sqcol, sqirow, sqicol;

	remap_coords(col, row, sqcol, sqrow, sqicol, sqirow);
	return squares[sqcol][sqrow].square_data[sqicol][sqirow];
}

bool SudokuBoard::is_valid(int col, int row, int value) const
{
	int sqrow, sqcol, sqirow, sqicol;

	remap_coords(col, row, sqcol, sqrow, sqicol, sqirow);

	// First make sure that value isn't already present in the square
	if(squares[sqcol][sqrow].is_present(value))
		return false;

	// Make sure it's not already present at any other spot in the row
	// or column
	for(int i = 0; i < SQUARE_INDEX_MAX; i++)
	{
		if(get_value(col, i) == value)
			return false;
		if(get_value(i, row) == value)
			return false;
	}

	return true;
}

bool SudokuBoard::is_valid()
{
	for(int col = 0; col < SQUARE_INDEX_MAX; col++)
	{
		for(int row = 0; row < SQUARE_INDEX_MAX; row++)
		{
			int value = get_value(col, row);
			// HACK: is_valid() will freak out at seeing the value already
			// in the board, so temporarily set it to be blank
			set_value(col, row, -1);
			if(value != -1 && !is_valid(col, row, value))
			{
				std::cout << "Invalid value at " << col << "," << row << std::endl;
				return false;
			}
			set_value(col, row, value);
		}
	}
	return true;
}

void SudokuBoard::read(char *filename)
{
	std::fstream infile(filename, std::ios_base::in);

	if(!infile)
	{
		std::cout << "Failed to open the input file." << std::endl;
		throw -1;
	}

	for(int row = 0; row < SQUARE_INDEX_MAX; row++)
	{
		for(int col = 0; col < SQUARE_INDEX_MAX; col++)
		{
			int value;
			infile >> value;
			set_value(col, row, value);
		}
	}
}

void SudokuBoard::print()
{
	for(int row = 0; row < SQUARE_INDEX_MAX; row++)
	{
		for(int col = 0; col < SQUARE_INDEX_MAX; col++)
		{
			int value = get_value(col, row);
			if(value == -1)
				printf(" - ");
			else
				printf("%2d ", value);
			if(col % SQUARE_SIZE == SQUARE_SIZE - 1)
				printf(" ");
		}
		printf("\n");
		if(row % SQUARE_SIZE == SQUARE_SIZE - 1)
			printf("\n");
	}
}

void SudokuBoard::remap_coords(int gcol, int grow, int& sqcol, int& sqrow, int& sqicol, int& sqirow) const
{
	sqcol = gcol / SQUARE_SIZE;
	sqrow = grow / SQUARE_SIZE;
	sqicol = gcol % SQUARE_SIZE;
	sqirow = grow % SQUARE_SIZE;

	if(sqcol >= SQUARE_SIZE || sqrow >= SQUARE_SIZE ||
           sqicol >= SQUARE_SIZE || sqirow >= SQUARE_SIZE)
	{
		throw -1;
	}
}

void SudokuBoard::next_empty(int& col, int& row)
{
	for(col = 0; col < SQUARE_INDEX_MAX; col++)
	{
		for(row = 0; row < SQUARE_INDEX_MAX; row++)
		{
			if(get_value(col, row) == -1)
				return;
		}
	}

	throw -1;
}

// If the board is valid and there are no empty cells, then the board is solved
bool SudokuBoard::is_solved()
{
	if(!is_valid())
		return false;

	bool solved = true;
	int col, row;
	for(col = 0; col < SQUARE_INDEX_MAX; col++)
	{
		for(row = 0; row < SQUARE_INDEX_MAX; row++)
		{
			if(get_value(col, row) == -1)
				solved = false;
		}
	}

	return solved;
}
